﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Threading.Tasks;
using UnityEngine;
using FirebaseREST;

public class DatabaseHandler : MonoBehaviour
{
    [SerializeField]
    private string databaseUrl;

    public static DatabaseHandler Instance;

    void Awake()
    {
        if (Instance == null)
        {
            Instance = this;
        }
        else if (Instance != this)
        {
            Destroy(gameObject);
        }

        DontDestroyOnLoad(gameObject);
    }

    void Start()
    {
        Initialize();
    }

    private void Initialize()
    {
    }

    public Action LoadScores(Action<Score, string> onValueAdded, int limit = 10)
    {
        EventHandler<ValueChangedEventArgs> callback = (sender, args) =>
        {
            if (args.DatabaseError != null)
            {
                Debug.LogError(args.DatabaseError.Message);
                return;
            }

            if (onValueAdded != null)
            {
                var snapshot = args.Snapshot;

                var data =  snapshot.Value as Dictionary<string, object>;
                Debug.Assert(data != null);

                int count = 0;
                foreach (var pair in data)
                {
                    count++;
                    if (args.ChangedKey.Equals(DatabaseReference.ALLKEYS) || args.ChangedKey.Equals(pair.Key))
                    {
                        var dict = pair.Value as Dictionary<string, object>;
                        var scoreObj = new Score();

                        scoreObj.OverwriteFromDictionary(dict);

                        scoreObj.id = pair.Key;
                        onValueAdded(scoreObj, count >= data.Count? args.ChangedKey : null);
                    }
                }
            }
        };

        var reference = FirebaseDatabase.Instance.GetReference("scores")
                            .OrderByChild("score")
                            .LimitToLast(limit);

        reference.ValueChanged += callback;

        Action unsubscribe = () =>
        {
            reference.ValueChanged += callback;
        };

        return unsubscribe;
    }

    public void GetScoreCount(Action<long> onComplete)
    {
        var reference = FirebaseDatabase.Instance.GetReference("scores");
        reference.GetValueAsync(10, res => {
            var snapshot = res.data;
            Debug.Log("res:" + snapshot.Children.Count);
            onComplete(snapshot.Children.Count);
        });
    }

    public void PostScore(Score score, Action onComplete = null, Action onError = null)
    {
        string json = JsonUtility.ToJson(score);

        var reference = FirebaseDatabase.Instance.GetReference("scores");

        reference.PushJson(json, 10, (res) => {
        if (res.success)
            {
                Debug.Log("Pushed with id: " + res.data);
                if (onComplete != null) onComplete();
            }
            else
            {
                Debug.Log("Push failed : " + res.message);
                if (onError != null) onError();
            }
        });
    }
}
