﻿using System.Collections;
using System.Collections.Generic;
using System;
using UnityEngine;

public class BGScroller : MonoBehaviour
{
    
    public PlayerMovement player;
    public Transform cameraTransform;
    public GameObject bgPrefab;
    public GameObject bgPrefabRoot;
    public Dictionary<Tuple<int, int>, GameObject> bgSet;
    public float bgHeight;
    public float bgWidth;

    public float parallaxFactor;

    private Vector3 startCameraPos;
    private Vector3 startPos;

    void Start()
    {
        bgPrefab.SetActive(false);
        bgSet = new Dictionary<Tuple<int, int>, GameObject>();

        startCameraPos = cameraTransform.position;
        startPos = transform.position;
    }

    void Update()
    {
        int gridPosX = Mathf.RoundToInt(player.transform.position.x / bgWidth);
        int gridPosY = Mathf.RoundToInt(player.transform.position.y / bgHeight);
        Tuple<int, int> gridPos = new Tuple<int, int>(gridPosX, gridPosY);
        CheckTile(new Tuple<int, int>(gridPosX, gridPosY));
        CheckTile(new Tuple<int, int>(gridPosX, gridPosY-1));
        CheckTile(new Tuple<int, int>(gridPosX, gridPosY+1));
        CheckTile(new Tuple<int, int>(gridPosX-1, gridPosY));
        CheckTile(new Tuple<int, int>(gridPosX-1, gridPosY-1));
        CheckTile(new Tuple<int, int>(gridPosX-1, gridPosY+1));
        CheckTile(new Tuple<int, int>(gridPosX+1, gridPosY));
        CheckTile(new Tuple<int, int>(gridPosX+1, gridPosY-1));
        CheckTile(new Tuple<int, int>(gridPosX+1, gridPosY+1));
    }

    void CheckTile(Tuple<int, int> gridPos)
    {
        if(!bgSet.ContainsKey(gridPos))
        {
            GameObject newBg = Instantiate(bgPrefab, bgPrefabRoot.transform, false);
            Vector3 offset = new Vector3(gridPos.Item1 * bgWidth, gridPos.Item2 * bgHeight, 0);
            newBg.transform.position = bgPrefabRoot.transform.position + offset;
            newBg.SetActive(true);
            bgSet.Add(gridPos, newBg);
        }

    }

    private void LateUpdate()
    {
        var position = startPos;

        position += parallaxFactor * (cameraTransform.position - startCameraPos);

        transform.position = position;
    }
}
