﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class BranchSpawner : MonoBehaviour
{
    public int age;

    public Vector3 growthDirection;

    [SerializeField]
    private LineRenderer lineRenderer;

    [SerializeField]
    private float speed;

    [SerializeField]
    private float length;

    [SerializeField]
    private float width;

    [SerializeField]
    private float widthDecay;

    [SerializeField]
    private float lerpFactor;

    [SerializeField]
    private float interval;

    [SerializeField]
    private float jitter;

    private bool isMoving;
    private Vector3 currentPosition;
    private Vector3 targetPosition;
    private Vector3 lastPoint;
    private int pointCounter;
    private Quaternion[] rotators;

    private Quaternion[] normals;
    private Vector3[] jitterEndpoints;
    private Vector3 jitterPos;

    private int childCount;

    private List<int> branchPoints;


    void Awake()
    {
        currentPosition = transform.position;
        lastPoint = currentPosition;
        lineRenderer.positionCount = 2;
        lineRenderer.SetPosition(0, lastPoint);
        lineRenderer.SetPosition(1, lastPoint);
        pointCounter = 1;
        rotators = new Quaternion[]{ Quaternion.Euler(0, 0, 20f), Quaternion.Euler(0, 0, -20f) };
        normals = new Quaternion[] { Quaternion.Euler(0, 0, 90f), Quaternion.Euler(0, 0, -90f) };
        jitterEndpoints = new Vector3[] { normals[0] * growthDirection * jitter, normals[1] * growthDirection  * jitter};
        
    }

    public void Grow()
    {
        if (age <= 0)
        {
            Destroy(gameObject);
            return;
        }

        targetPosition = transform.position + (growthDirection * age * length);
        childCount = age > 1? Random.Range(1, 3) : 0;
        branchPoints = GetRandomPoints(childCount);

        // lineRenderer.sortingOrder = Random.Range(-5, 5);
        lineRenderer.startWidth = width;
        lineRenderer.endWidth = width;

        isMoving = true;
    }

    // Update is called once per frame
    void Update()
    {
        if (Input.GetKeyDown(KeyCode.Q)) Grow();
        if (!isMoving) return;

        currentPosition = Vector3.Lerp(currentPosition, targetPosition + jitterPos, lerpFactor * Time.deltaTime);

        var newPos = currentPosition;
        lineRenderer.SetPosition(pointCounter, newPos);

        if (Vector3.Distance(lastPoint, newPos) > interval)
        {
            lineRenderer.positionCount++;

            lineRenderer.SetPosition(pointCounter++, newPos);
            lineRenderer.SetPosition(pointCounter, newPos);

            jitterPos = Vector3.Lerp(jitterEndpoints[0], jitterEndpoints[1], Random.value);
            
            lastPoint = newPos;
        }

        if (branchPoints.Count > 0)
        {
            if ((currentPosition - transform.position).magnitude > age * length * branchPoints[0] * 0.1f)
            {
                var branch = Instantiate(this, currentPosition, Quaternion.identity);
                branch.name = "Branch";
                branch.transform.parent = transform.parent;
                branch.age = age - 1;
                branch.width = width * widthDecay;
                branch.growthDirection = rotators[--childCount] * growthDirection;
                branchPoints.RemoveAt(0);
                branch.Grow();
            }
        }

        if (Vector2.SqrMagnitude(currentPosition - targetPosition) <= 0.01f)
        {
            isMoving = false;
            enabled = false;
        }
    }

    private List<int> GetRandomPoints(int count)
    {
        var lottery = new List<int>(new int[] {4, 5, 6, 7, 8});

        if (count == 0)
        {
            lottery.Clear();
            return lottery;
        }

        var toRemove = lottery.Count - count;

        while (toRemove-- > 0)
        {
            lottery.RemoveAt(Random.Range(0, lottery.Count));
        }

        return lottery;
    }
}
