﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class DespawnManager : MonoBehaviour
{
    [SerializeField]
    private float topBound;
    [SerializeField]
    private float sideBound;

    [SerializeField]
    private Transform cameraTransform;

    public static DespawnManager instance;

    private Queue<Despawnable> despawnQueue = new Queue<Despawnable>();
    
    private object lockObj = new object();

    void Awake()
    {
        if (instance == null)
        {
            instance = this;
        }
    }

    public void Add(Despawnable item)
    {
        lock(lockObj)
        {
            despawnQueue.Enqueue(item);
        }
    }

    void Update()
    {
        if (Time.frameCount % 4 == 0)
        {
            lock(lockObj)
            {
                if (despawnQueue.Count <= 0) return;

                var item = despawnQueue.Peek();

                if (item == null)
                {
                    despawnQueue.Dequeue();
                    return;
                }

                if (item.transform.position.x - cameraTransform.position.x < -sideBound ||
                    item.transform.position.x - cameraTransform.position.x > sideBound ||
                    item.transform.position.y - cameraTransform.position.y > topBound)
                {
                    Destroy(despawnQueue.Dequeue().gameObject);
                }
            }
            
        }
    }

}
