﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[CreateAssetMenu(menuName = "Events/Game Event")]
public class GameEvent : ScriptableObject
{
    private object m_lock = new object();

    private List<GameEventListener> listeners = new List<GameEventListener>();

    public void AddListener(GameEventListener listener)
    {
        if (!this.listeners.Contains(listener))
        {
            //Debug.Log("Adding " + listener.name);
            this.listeners.Add(listener);
        }
    }

    public void RemoveListener(GameEventListener listener)
    {
        if (this.listeners.Contains(listener))
        {
            //Debug.Log("Removing " + listener.name);
            this.listeners.Remove(listener);
        }
    }

    public void Raise()
    {
        Debug.Log(this.name + " event raised");

        lock (m_lock)
        {
            foreach (GameEventListener listener in this.listeners)
            {
                listener.OnRaiseEvent();
                Debug.Log(listener.name + " listener reacted");
            }
        }
    }
}
