﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using EZCameraShake;
using System.IO;

public class GameManager : MonoBehaviour
{
    public static GameManager instance;

    [SerializeField]
    private Player player;

    [Space(5)]
    [SerializeField]
    private SeedneyController seedneyController;

    [Header("Depth Stuff")]
    [SerializeField]
    private string depthUnit = "m";
    [SerializeField]
    private float depthScale = 0.5f;

    [Header("Managers")]
    [SerializeField]
    private PickupsManager pickupsManager;

    [Header("UI")]
    [SerializeField]
    private MainGameUI mainGameUI;

    [Header("FX")]
    [SerializeField]
    private CameraShaker cameraShaker;
    [SerializeField]
    private float shakeMagnitude;
    [SerializeField]
    private float shakeRoughness;
    [SerializeField]
    private float shakeFadeInTime;
    [SerializeField]
    private float shakeFadeOutTime;

    [SerializeField]
    private TreeDataSet dataSet;

    private string userName;
    public string UserName 
    { 
        get { return userName; } 
        set { userName = value; }
    }

    public string DepthUnit { get { return depthUnit; } }
    public float DepthScale { get { return depthScale; } }

    public Player Player { get { return player; } }
    public SeedneyController SeedneyController { get { return seedneyController; } }
    public PickupsManager PickupsManager { get { return pickupsManager; } }
    public MainGameUI MainGameUI { get { return mainGameUI; } }
    
    private CameraShakeInstance shakeInstance;

    void Awake()
    {
        if (instance == null)
        {
            instance = this;
        }

        Init();
    }

    public void Init()
    {
        mainGameUI.Init();
        player.Init();
        seedneyController.Init();

        pickupsManager.OnPickupGet += OnPickupGet;

        player.OnDeath += OnDeath;
    }

    void OnDestroy()
    {
        pickupsManager.OnPickupGet -= OnPickupGet;
        player.OnDeath -= OnDeath;
    }

    public float GetDepth(bool applyScaling = true)
    {
        float scaling = 1.0f;
        if (applyScaling)
        {
            scaling = depthScale;
        }

        return -player.transform.position.y * scaling;
    }

    private void OnDeath()
    {
        var score = new Score();
        score.name = NameStorage.UserName;
        score.score = (long)GetDepth();
        score.treeId = Random.Range(0, dataSet.data.Length);
        score.pos = Random.Range(-dataSet.range, dataSet.range);

        var json = JsonUtility.ToJson(score);

        ScoreManager.instance.PostScore(score);

        //File.WriteAllText(Path.Combine(Application.persistentDataPath, "score.json"), json);
        PlayerPrefs.SetString("score", json);
        PlayerPrefs.Save();

        SceneManager.instance.LoadScene("ForestView", Color.white, 1f);
    }

    public void OnHazardEntered()
    {
        //Debug.Log("Boom", CameraShaker.Instance.gameObject);
        // cameraShaker.ShakeOnce(shakeMagnitude, shakeRoughness, shakeFadeInTime, shakeFadeOutTime);
        CameraShaker.Instance.ShakeOnce(shakeMagnitude, shakeRoughness, shakeFadeInTime, shakeFadeOutTime);
        seedneyController.OnHazardEnter();
    }

    public void OnHazardExited()
    {
        seedneyController.OnHazardExit();
    }

    public void OnPickupGet(Pickup pickup)
    {
        seedneyController.OnPickupGet(pickup);
    }
}
