﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class SceneManager : MonoBehaviour
{
    public static SceneManager instance;

    public Image fader;
    public AnimationCurve faderCurve;

    void Awake()
    {
        if (instance == null)
        {
            instance = this;
        }
        else if (instance != this)
        {
            Destroy(gameObject);
        }

        DontDestroyOnLoad(gameObject);
        
        fader.canvasRenderer.SetAlpha(0);
        fader.enabled = true;
    }
    
    public void LoadScene(string name, Color color, float duration, AnimationCurve curve = null)
    {
        StartCoroutine(LoadSceneCoroutine(name, color, duration, curve?? faderCurve));
    }

    private IEnumerator LoadSceneCoroutine(string name, Color color, float duration, AnimationCurve curve = null)
    {
        var timer = 0f;

        fader.color = color;

        var currentScene = UnityEngine.SceneManagement.SceneManager.GetActiveScene();

        while (timer < duration)
        {
            fader.canvasRenderer.SetAlpha(curve.Evaluate(timer / duration));
            timer += Time.deltaTime;
            yield return new WaitForEndOfFrame();
        }
        fader.canvasRenderer.SetAlpha(curve.Evaluate(1f));
        var loadAsync = UnityEngine.SceneManagement.SceneManager.LoadSceneAsync(name, UnityEngine.SceneManagement.LoadSceneMode.Additive);
        yield return loadAsync;

        var unloadAsync = UnityEngine.SceneManagement.SceneManager.UnloadSceneAsync(currentScene.buildIndex);
        yield return unloadAsync;

        // while (!loadAsync.isDone || !unloadAsync.isDone)
        // {
        //     yield return null;
        // }

        timer = duration;
        while (timer > 0)
        {
            fader.canvasRenderer.SetAlpha(curve.Evaluate(timer / duration));
            timer -= Time.deltaTime;
            yield return new WaitForEndOfFrame();
        }

        fader.canvasRenderer.SetAlpha(curve.Evaluate(0f));
    }
}
