﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class Player : MonoBehaviour
{
    [SerializeField]
    private PlayerMovement movement;

    [SerializeField]
    private PlayerRootRenderer rootRenderer;

    [SerializeField]
    private float maxGrowthAmt = 100.0f;

    [SerializeField]
    private float decayRate = 2.0f;

    private bool doUpdate = false;
    private bool isAlive = false;

    private float growthAmt = 100.0f;
    private float decayDelta = 0;

    public PlayerMovement Movement { get { return movement; } }
    public PlayerRootRenderer RootRenderer { get { return rootRenderer; } }
    public float MaxGrowthAmt { get { return maxGrowthAmt; } }

    public bool IsAlive { get { return isAlive; } }

    public System.Action OnDeath;

    public float GetGrowthAmt()
    {
        return growthAmt;
    }


    public void Init()
    {
        growthAmt = maxGrowthAmt;
        isAlive = true;
        Unpause();
    }

    public void Pause()
    {
        doUpdate = false;
    }

    public void Unpause()
    {
        doUpdate = true;
    }

    public void Update()
    {
        if (!doUpdate) return;
        if (!isAlive) return;

        growthAmt -= (decayRate + decayDelta) * Time.deltaTime;
        if (growthAmt <= 0)
        {
            growthAmt = 0;
            if (OnDeath != null)
            {
                Die();
            }
            isAlive = false;
        }
    }

    private void Die()
    {
        GameManager.instance.SeedneyController.OnDeath();
        StartCoroutine(DieCoroutine());
    }

    private IEnumerator DieCoroutine()
    {
        Movement.StartDecay();
        while (Movement.isDying)
        {
            yield return null;
            if (Movement.hasStopped)
            {
                if (OnDeath != null)
                    OnDeath();
                yield break;
            }
        }

    }

    public void SetDecayDelta(float value)
    {
        decayDelta = value;
    }

    public void AddDecayDelta(float value)
    {
        decayDelta += value;
    }

    public void SetGrowthAmt(float value)
    {
        growthAmt = value;
    }

    public void AddGrowthAmt(float value)
    {
        growthAmt += value;
        if (growthAmt > maxGrowthAmt)
        {
            growthAmt = maxGrowthAmt;
        }
    }
}
