﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(Collider2D))]
public class Pickup : MonoBehaviour
{
    [SerializeField]
    private bool isHazard = false;

    [Header("Ongoing Effects")]
    [SerializeField]
    private float duration = 3.0f;
    [SerializeField]
    private float horizontalDelta = 0;
    [SerializeField]
    private float verticalDelta = 0;
    [SerializeField]
    private float decayRateDelta = 0;

    [Header("Instant effects")]
    [SerializeField]
    private bool isInstant = false;
    [SerializeField]
    private float growthAmtDelta = 0;

    [Header("FX on Pickup Get")]
    [SerializeField]
    private int burstBranchGrowMin = 3;
    [SerializeField]
    private int burstBranchGrowMax = 6;
    [SerializeField]
    private float burstBranchIntervalMin = 0.05f;
    [SerializeField]
    private float burstBranchIntervalMax = 0.01f;

    [Header("SFX")]
    [SerializeField]
    private AudioSource pickupGetSFXSource = null;

    private PickupsManager manager = null;

    private Animator animator = null;

    public bool IsHazard { get { return isHazard; } }
    public bool IsInstant { get { return isInstant; } }
    public float Duration { get { return duration; } }
    public float HorizontalDelta { get { return horizontalDelta; } }
    public float VerticalDelta { get {return verticalDelta; } }
    public float DecayRateDelta { get { return decayRateDelta; } }
    public float GrowthAmtDelta { get { return growthAmtDelta; } }

    public int BurstBranchGrowMin { get { return burstBranchGrowMin; } }
    public int BurstBranchGrowMax { get { return burstBranchGrowMax; } }

    public float BurstBranchIntervalMin { get { return burstBranchIntervalMin; } }
    public float BurstBranchIntervalMax { get { return burstBranchIntervalMax; } }

    public void Init(PickupsManager parentManager)
    {
        manager = parentManager;
        animator = GetComponent<Animator>();
    }

    void OnTriggerEnter2D(Collider2D col)
    {
        GameObject otherColliderObj = col.gameObject;

        if (otherColliderObj.tag != "Player")
        {
            return; // ignore
        }
        
        if (manager == null)
        {
            manager = GameManager.instance.PickupsManager;

            if (manager == null)
            {
                return;
            }
        }

        Player player = otherColliderObj.GetComponent<Player>();
        if (player == null)
        {
            return;
        }

        manager.ApplyPickup(this, player);
        this.transform.localScale *= 0.5f;
        this.GetComponent<Collider2D>().enabled = false;
        PlayAnim();
        PlaySound();
    }

    private void PlayAnim()
    {
        if (animator == null)
        {
            animator = GetComponent<Animator>();
            if (animator == null) return;
        }

        animator.SetTrigger("Burst");
    }

    private void PlaySound()
    {
        if (pickupGetSFXSource == null) return;
        if (pickupGetSFXSource.clip == null) return;

        pickupGetSFXSource.Play();
    }

    public void AnimDone()
    {
        Destroy(gameObject);
    }
}
