﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

// Manages positive and negative pickups (Powerups & Hazards)
public class PickupsManager : MonoBehaviour
{
    // settings asset file here
    [SerializeField]
    private PickupsManagerConfig defaultConfig;

    // actual params we'll be using
    private List<Pickup> powerupPickups;
    private List<Pickup> hazardPickups;

    public System.Action<Pickup> OnPickupGet;

    private void Awake()
    {
        if (defaultConfig == null)
        {
            Debug.LogWarning("default pickups config not set. the game will now explode");
            return;
        }

        powerupPickups = defaultConfig.GetPowerups();
        hazardPickups = defaultConfig.GetHazards();
    }

    public void ApplyPickup(Pickup pickup, Player player)
    {
        if (pickup == null || player == null)
        {
            Debug.LogWarning("Pickup or player is null, not applying pickup effect");
            return;
        }

        StartCoroutine(ApplyPickupCR(pickup, player));
    }

    private IEnumerator ApplyPickupCR(Pickup pickup, Player player)
    {
        float duration = pickup.Duration;
        float timer = 0;

        if (OnPickupGet != null)
        {
            OnPickupGet(pickup);
        }

        if (pickup.IsInstant)
        {   // apply instant effects
            player.AddGrowthAmt(pickup.GrowthAmtDelta);
            yield break;
        }
        else
        {
            // apply effects
            player.Movement.vSpeed += pickup.VerticalDelta;
            player.Movement.hSpeed += pickup.HorizontalDelta;
            player.AddDecayDelta(pickup.DecayRateDelta);

            // wait
            while (true)
            {
                timer += Time.deltaTime;
                if (timer >= duration)
                {
                    break;
                }

                yield return new WaitForEndOfFrame();
            }

            // remove effects
            player.Movement.vSpeed -= pickup.VerticalDelta;
            player.Movement.hSpeed -= pickup.HorizontalDelta;
            player.AddDecayDelta(-pickup.DecayRateDelta);
        }
        yield return null;
    }
}
