﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PlayerRootRenderer : MonoBehaviour
{
    [SerializeField]
    private LineRenderer lineRenderer;

    [SerializeField]
    private BranchSpawner branchPrefab;

    [SerializeField]
    private float interval;

    [SerializeField]
    private float branchInterval;
    
    [SerializeField]
    private Vector3 offset;

    [SerializeField]
    private Transform target;

    private Vector3 lastPoint;
    private int pointCounter;
    private Vector3 previousPosition;

    private Vector3 startPoint;
    private float currentDistance;
    private float targetDistance;
    private Quaternion[] branchGrowthDirs;

    // Start is called before the first frame update
    void Start()
    {
        Init();
    }

    void OnDestroy()
    {
        GameManager.instance.PickupsManager.OnPickupGet -= OnPickupGet;
    }

    private void Init()
    {
        lastPoint = target.position + offset;
        previousPosition = target.position;
        lineRenderer.positionCount = 2;
        lineRenderer.SetPosition(0, lastPoint);
        lineRenderer.SetPosition(1, lastPoint);
        pointCounter = 1;
        targetDistance = branchInterval;
        startPoint = transform.position;

        branchGrowthDirs = new Quaternion[]{ Quaternion.Euler(0, 0, -30f), Quaternion.Euler(0, 0, 30f) };

        GameManager.instance.PickupsManager.OnPickupGet += OnPickupGet;
    }

    // Update is called once per frame
    void FixedUpdate()
    {
        var newPos = target.position + offset;
        lineRenderer.SetPosition(pointCounter, newPos);

        currentDistance = Vector3.Distance(startPoint, target.position);
        if (currentDistance >= targetDistance)
        {
            targetDistance += branchInterval;
            var branch = Instantiate(branchPrefab, target.position, Quaternion.identity);
            branch.growthDirection = (branchGrowthDirs[Random.Range(0,2)] * (target.position - previousPosition)).normalized;
            branch.Grow();
        }

        if (Vector3.Distance(lastPoint, target.position) > interval)
        {
            lineRenderer.positionCount++;

            lineRenderer.SetPosition(pointCounter++, newPos);
            lineRenderer.SetPosition(pointCounter, newPos);

            lastPoint = target.position;
        }

        previousPosition = target.position;
    }

    public void OnPickupGet(Pickup pickup)
    {
        if (pickup == null) return;
        if (pickup.IsHazard) return;
        
        StartCoroutine(BurstBranchGrow( Random.Range(pickup.BurstBranchGrowMin, pickup.BurstBranchGrowMax), 
                                        pickup.BurstBranchIntervalMin, 
                                        pickup.BurstBranchIntervalMax));
    }

    private IEnumerator BurstBranchGrow(int amount, float intervalMin = 0.05f, float intervalMax = 0.1f)
    {
        for (int i = 0; i < amount; i++)
        {
            var branch = Instantiate(branchPrefab, target.position, Quaternion.identity);
            branch.growthDirection = (branchGrowthDirs[Random.Range(0,2)] * (target.position - previousPosition)).normalized;
            branch.Grow();

            yield return new WaitForSeconds(Random.Range(intervalMin, intervalMax));
        }
        yield return null;
    }
}
