﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class ScoreManager : MonoBehaviour
{
    public static ScoreManager instance;

    public bool scoreSubscribed { get { return handlerUnsubscribe != null; } }
    public bool scoreInitialized { get; private set;}
    
    public static Action<Score> onScoreUpdated;

    private Action handlerUnsubscribe;

    public List<Score> scoreList = new List<Score>();

    void Awake()
    {
        if (instance == null)
        {
            instance = this;
        }
        else if (instance != this)
        {
            Destroy(gameObject);
        }

        DontDestroyOnLoad(gameObject);
    }

    void Update()
    {
        if (Input.GetKeyDown(KeyCode.W))
        {
            LoadScores();
        }
        if (Input.GetKeyDown(KeyCode.E))
        {
            Score score = new Score();
            score.name = "Test" + Guid.NewGuid().ToString();
            //NameStorage storage = NameStorage.Instance;
            score.name = NameStorage.UserName + Guid.NewGuid().ToString();
 
            score.score = UnityEngine.Random.Range(1000, 100000);
            PostScore(score);
        }
        if (Input.GetKeyDown(KeyCode.R))
        {
            foreach (var score in scoreList)
            {
                Debug.LogFormat("{0} : {1}", score.name, score.score);
            }
        }
    }

    public void LoadScores()
    {
        if (handlerUnsubscribe != null) handlerUnsubscribe();
        handlerUnsubscribe = DatabaseHandler.Instance.LoadScores(OnScoreUpdated, 1000);
    }

    private void OnScoreUpdated(Score score, string changedKey)
    {
        Debug.LogFormat("{0} {1} {2}", score.id, score.name, score.score);

        var index = scoreList.BinarySearch(score);
        if (index < 0)
        {
            scoreList.Insert(~index, score);
        }

        if (changedKey != null && changedKey.Equals(FirebaseREST.DatabaseReference.ALLKEYS))
        {
            scoreInitialized = true;
        }

        if (onScoreUpdated != null)
        {
            onScoreUpdated(score);
        }
    }

    public void PostScore(Score score)
    {
        DatabaseHandler.Instance.PostScore(score);
    }
}
