﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class ScreenSystemManager : MonoBehaviour
{
    public static ScreenSystemManager Instance { get; private set; }

    public ScreenSetAsset screenSet;
    public string firstScreen;

    private Stack<ScreenAsset> history = new Stack<ScreenAsset>();

    public GameObject currentScreenGameObject { get; private set; }

    private string currentScreenName = null;
    public string CurrentScreenName{ get { return currentScreenName; } }

    void Awake()
    {
        if (Instance == null)
        {
            Instance = this;
        }
    }

    void Start()
    {
        if (!string.IsNullOrEmpty(firstScreen))
        {
            SetScreen(firstScreen);
        }
    }

    public void SetScreen(string name, bool overlay = false)
    {
        var screen = screenSet.Find(name);

        SetScreen(screen);
        currentScreenName = name;
    }

    public void SetScreen(ScreenAsset screen, bool overlay = false)
    {
        Debug.Assert(screen != null);

        if (screen != null)
        {
            var go = Instantiate(screen.prefab);

            if (currentScreenGameObject != null)
            {
                Destroy(currentScreenGameObject);
            }

            currentScreenGameObject = go;
            history.Push(screen);
        }
    }

    public void Back()
    {
        if (history.Count <= 1)
            return;

        history.Pop();
        var screen = history.Peek();

        Debug.Log(screen.name);

        var go = Instantiate(screen.prefab);

        if (currentScreenGameObject != null)
        {
            Destroy(currentScreenGameObject);
        }

        currentScreenGameObject = go;
    }
}
