﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Tree : MonoBehaviour
{
    [SerializeField]
    private SpriteRenderer sprite;

    [SerializeField]
    private SpriteRenderer mid;

    [SerializeField]
    private SpriteRenderer top;

    [SerializeField]
    private Transform treeGroup;

    [SerializeField]
    private AnimationCurve curveX;
    [SerializeField]
    private AnimationCurve curveY;

    [SerializeField]
    private float duration;

    [SerializeField]
    private float height;

    [SerializeField]
    private SpriteRenderer[] sprites;

    private float timer;
    private bool isGrowing;

    public void SetHeight(float height)
    {
        this.height = height;
        //sprite.size = new Vector2(sprite.size.x, height);
    }

    public void SetColor(Color col)
    {
        foreach (var item in sprites)
        {
            item.color = col;
        }
    }

    public void SetSortingIndex(int index)
    {
        foreach (var item in sprites)
        {
            item.sortingOrder = index;
        }
    }

    public void LoadData(TreeData data)
    {
        for (int i = 0; i < sprites.Length; i++)
        {
            sprites[i].sprite = data.spriteList[i];
        }
    }

    private float GetMidHeight()
    {
        return mid.sprite.textureRect.height;
    }

    void Update()
    {
        if (Input.GetKeyDown(KeyCode.Z)) StartGrowing();
        top.transform.localPosition = new Vector2(0, GetMidHeight() * mid.transform.localScale.y / mid.sprite.pixelsPerUnit);

        if (!isGrowing) return;
        var scaleY = height * curveY.Evaluate(timer / duration);
        var scaleX = curveX.Evaluate(timer / duration);
        mid.transform.localScale = new Vector3(1f, scaleY, 1f);
        sprite.transform.localScale = new Vector3(scaleX, 1f, 1f);
        timer += Time.deltaTime;

        if (timer > duration)
        {
            isGrowing = false;
        }
    }

    public void StartGrowing()
    {
        timer = 0;
        isGrowing = true;
    }
}
