﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class MainGameUI : MonoBehaviour
{
    [SerializeField]
    private Image growthBarValue = null;

    [SerializeField]
    private Text depthText = null;
    [SerializeField]
    [Range(0, 8)]
    private int depthTextPrecision = 1;

    private bool doUpdate = false;
    private GameManager gameManager;
    private Player player = null;

    public void Pause()
    {
        doUpdate = false;
    }

    public void Unpause()
    {
        doUpdate = true;
    }

    public void Init()
    {
        gameManager = GameManager.instance;
        player = GameManager.instance.Player;
        Unpause();
    }

    void Update()
    {
        if (!doUpdate) return;
        UpdateGrowthBar();
        UpdateDepthDisplay();
    }

    private void UpdateGrowthBar()
    {
        if (growthBarValue == null) return;

        growthBarValue.fillAmount = player.GetGrowthAmt() / player.MaxGrowthAmt;
    }

    private void UpdateDepthDisplay()
    {
        if (depthText == null) return;

        float cappedDepth = gameManager.GetDepth();
        if(cappedDepth < 1.0f)
        {
            cappedDepth = (cappedDepth + 4.0f) / 5f;
        }
        depthText.text = cappedDepth.ToString("F" + depthTextPrecision.ToString()) + " " + gameManager.DepthUnit;
    }
}
